import Clutter from 'gi://Clutter';
import Meta from 'gi://Meta';
import { Extension } from 'resource:///org/gnome/shell/extensions/extension.js';
const BACKGROUND_FACTOR = 'desaturation-factor-background';
const FOREGROUND_FACTOR = 'desaturation-factor-foreground';
const EFFECT_NAME = 'window-desaturation@mkhl.codeberg.page';
export default class WindowDesaturation extends Extension {
    connections;
    focused_window;
    settings;
    enable() {
        this.settings = this.getSettings();
        for (const window of this.windows()) {
            this.handle(window);
        }
        this.connections = [
            global.display.connect('window-created', (_, window) => this.handle(window)),
            global.display.connect('focus-window', (_, window) => this.update_focus(window)),
        ];
        this.settings?.connect('changed', () => {
            for (const window of this.windows()) {
                this.refocus(window);
            }
        });
        this.update_focus(global.display.focus_window);
    }
    disable() {
        this.settings = undefined;
        for (const id of this.connections ?? []) {
            global.display.disconnect(id);
        }
        this.connections = undefined;
        for (const window of this.windows()) {
            this.clear(window);
        }
    }
    handle(window) {
        const actor = this.actor(window);
        if (!actor)
            return;
        const factor = this.background_factor();
        const effect = new Clutter.DesaturateEffect({ factor });
        actor.add_effect_with_name(EFFECT_NAME, effect);
    }
    clear(window) {
        const actor = this.actor(window);
        if (!actor)
            return;
        actor.remove_effect_by_name(EFFECT_NAME);
    }
    update_focus(window) {
        if (this.focused_window === window)
            return;
        this.focus(window);
        this.defocus(this.focused_window);
        this.focused_window = window;
    }
    focus(window) {
        const effect = this.effect(window);
        if (!effect)
            return;
        effect.factor = this.foreground_factor();
    }
    defocus(window) {
        const effect = this.effect(window);
        if (!effect)
            return;
        effect.factor = this.background_factor();
    }
    refocus(window) {
        const effect = this.effect(window);
        if (!effect)
            return;
        effect.factor =
            this.focused_window === window
                ? this.foreground_factor()
                : this.background_factor();
    }
    actor(window) {
        if (!window)
            return;
        return window.get_compositor_private();
    }
    effect(window) {
        if (!window)
            return;
        const actor = this.actor(window);
        if (!actor)
            return;
        const effect = actor.get_effect(EFFECT_NAME);
        if (!(effect instanceof Clutter.DesaturateEffect))
            return;
        return effect;
    }
    windows() {
        return global.display
            .list_all_windows()
            .filter((it) => it.window_type === Meta.WindowType.NORMAL);
    }
    background_factor() {
        return this.settings.get_double(BACKGROUND_FACTOR);
    }
    foreground_factor() {
        return this.settings.get_double(FOREGROUND_FACTOR);
    }
}
